/*
 * vr.h.h
 Copyright (c) 1992 by Richard Mauri Trimble Navigation Ltd.

 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in
 supporting documentation.

 This file is provided AS IS with no warranties of any kind.  The author
 shall have no liability with respect to the infringement of copyrights,
 trade secrets or any patents by this file or any part thereof.  In no
 event will the author be liable for any lost revenue or profits or
 other special, indirect and consequential damages.
 */
#ifndef VR_H
#define VR_H

#include <stdio.h>
#include <string.h>
#include <Xm/ToggleB.h>
#include <Xm/RowColumn.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>

#include <Xm/TextF.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/Form.h>
#include <Xm/SeparatoG.h>

#define VRNAME "vr"

#define LABEL_LENGTH 20
#define STRING_LENGTH 32

typedef struct {
    int state;
    Window icon;
} WM_STATE;

typedef struct {
    int      room;
    Window    window, parent;
    WM_STATE  wm_state;
    char      name[80];
    int x,y,width,height;
} VRInfo;

#define UNSIGNEDINT unsigned long
#define SIGNEDINT long
#define RESOURCEID unsigned long
/* this structure may be extended, but do not change the order */

typedef struct {
    long state;
    unsigned long icon;
} xPropWMState;

#define NumPropWMStateElements 2 /* number of elements in this structure */

#define MAX_CLIENTS 250 /* This constant should be increased for shit-loads of shell windows */

typedef struct {
    Bool	verbose;
    Bool	want_dialog;
    String	vrLoadFile;
    String	vrTitleFile;
    String	vrAllRoomsFile;
    int		nrooms;
} AppData, *AppDataPtr;

typedef struct {
    Pixel	room_color;
    String	room_cmds;
    String	room_passwd;
} RoomData, *RoomDataPtr;
#define XtNnrooms "nrooms"
#define XtCnrooms "Nrooms"

#define XtNroom_cmds "room%d_cmds"
#define XtCroom_cmds "Room%d_cmds"

#define XtNroom_passwd "room%d_passwd"
#define XtCroom_passwd "Room%d_passwd"

#define XtNroomColor "room%dColor"
#define XtCroomColor "Room%dColor"

#define XtNvrLoadFile "vrLoadFile"
#define XtCvrLoadFile "VrLoadFile"

#define XtNvrTitleFile "vrTitleFile"
#define XtCvrTitleFile "VrTitleFile"

#define XtNvrAllRoomsFile "vrAllRoomsFile"
#define XtCvrAllRoomsFile "VrAllRoomsFile"

#define XtNverbose "verbose"
#define XtCverbose "Verbose"

#define XtNwant_dialog "want_dialog"
#define XtCwant_dialog "Want_dialog"

static XtResource resources[] = {
    {
	XtNwant_dialog,
	XtCwant_dialog,
	XtRBool,
	sizeof(Bool),
        XtOffset(AppDataPtr, want_dialog),
        XtRImmediate,
        (XtPointer) True,
    },
    {
	XtNnrooms,
	XtCnrooms,
	XtRInt,
	sizeof(int),
        XtOffset(AppDataPtr, nrooms),
        XtRImmediate,
        (XtPointer) 6,
    },
    {
	XtNvrAllRoomsFile,
	XtCvrAllRoomsFile,
	XtRString,
	sizeof(String),
        XtOffset(AppDataPtr, vrAllRoomsFile),
        XtRImmediate,
	"HOME/.vrallrooms"
    },
    {
	XtNvrTitleFile,
	XtCvrTitleFile,
	XtRString,
	sizeof(String),
        XtOffset(AppDataPtr, vrTitleFile),
        XtRImmediate,
	"HOME/.vrtitle"
    },
    {
	XtNvrLoadFile,
	XtCvrLoadFile,
	XtRString,
	sizeof(String),
        XtOffset(AppDataPtr, vrLoadFile),
        XtRImmediate,
	"HOME/.vrload"
    },
    {
	XtNverbose,
	XtCverbose,
	XtRBool,
	sizeof(Bool),
        XtOffset(AppDataPtr, verbose),
        XtRImmediate,
        (XtPointer) False,
    },
};

static XtResource room_resources[] = {
    {
	NULL,
	NULL,
	XtRPixel,
	sizeof(Pixel),
	XtOffset(RoomDataPtr, room_color),
	XtRString,
	"grey40",
    },
    {
	NULL,
	NULL,
	XtRString,
	sizeof(String),
        XtOffset(RoomDataPtr, room_cmds),
        XtRImmediate,
	NULL
    },
    {
	NULL,
	NULL,
	XtRString,
	sizeof(String),
        XtOffset(RoomDataPtr, room_passwd),
        XtRImmediate,
	NULL
    },
};

static XrmOptionDescRec options[] = {
   { "-room1_passwd",	"*room1_passwd",	XrmoptionSepArg, NULL },
   { "-room2_passwd",	"*room2_passwd",	XrmoptionSepArg, NULL },
   { "-room3_passwd",	"*room3_passwd",	XrmoptionSepArg, NULL },
   { "-room4_passwd",	"*room4_passwd",	XrmoptionSepArg, NULL },
   { "-room5_passwd",	"*room5_passwd",	XrmoptionSepArg, NULL },
   { "-room6_passwd",	"*room6_passwd",	XrmoptionSepArg, NULL },
   { "-verbose","*verbose",	XrmoptionNoArg, "true" },
   { "-fg",	"*foreground",	XrmoptionSepArg, NULL },
   { "-bg",	"*background",	XrmoptionSepArg, NULL },
   { "-room1_cmds",	"*room1_cmds",	XrmoptionSepArg, NULL },
   { "-room2_cmds",	"*room2_cmds",	XrmoptionSepArg, NULL },
   { "-room3_cmds",	"*room3_cmds",	XrmoptionSepArg, NULL },
   { "-room4_cmds",	"*room4_cmds",	XrmoptionSepArg, NULL },
   { "-room5_cmds",	"*room5_cmds",	XrmoptionSepArg, NULL },
   { "-room6_cmds",	"*room6_cmds",	XrmoptionSepArg, NULL },
   { "-r1c",	"*room1Color",	XrmoptionSepArg, NULL },
   { "-r2c",	"*room2Color",	XrmoptionSepArg, NULL },
   { "-r3c",	"*room3Color",	XrmoptionSepArg, NULL },
   { "-r4c",	"*room4Color",	XrmoptionSepArg, NULL },
   { "-r5c",	"*room5Color",	XrmoptionSepArg, NULL },
   { "-r6c",	"*room6Color",	XrmoptionSepArg, NULL },
   { "-vrLoadFile",	"*vrLoadFile",		XrmoptionSepArg, NULL },
   { "-vrTitleFile",	"*vrTitleFile",		XrmoptionSepArg, NULL },
   { "-vrAllRoomsFile",	"*vrAllRoomsFile",	XrmoptionSepArg, NULL },
   { "-nrooms",	"*nrooms",		XrmoptionSepArg, NULL },
   { "-nrows",	"*rc.numColumns",	XrmoptionSepArg, NULL },
   { "-dialog",	"*want_dialog",	XrmoptionNoArg, "False"},
   { "+dialog",	"*want_dialog",	XrmoptionNoArg, "True"},
};

static String fallbacks[] = {
   "Vr*initialResourcesPersistent: false",
   "Vr*background          : dodgerblue4",
   "Vr*fontList    	     : -*-lucida-bold-r-*-*-12-*-*-*-*-*-*-*",
   "Vr*XmToggleButton.indicatorOn     : false",
   "Vr*XmToggleButton.shadowThickness : 2",
   NULL
};

#endif
