/*
 * xpasswd.c
 *
 * Here's how to use it:
 *	first to create you password "database" do:
 *	xpasswd +set -passwd file_to_use &
 *		(Enter the password and it will be saved crypted in file_to_use)
 *	This file name including path (env vars OK) should be used with vr (see vr.man)
 *	or do something like: vr -room3_passwd $HOME/.vr3 (for passwd protection on room3)
 *
 * (This logic was largely taken from a recent article in the X journal)
 */
#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <Xm/Text.h>
#include <Xm/LabelG.h>
#include <Xm/RowColumn.h>
#include <Xm/BulletinB.h>
#include <ctype.h>

extern char *ExpandFileName();

static XtCallbackProc check_passwd();
static char *passwd;

static XrmOptionDescRec options[] = {
   { "-fg", 		"*foreground",	XrmoptionSepArg, NULL },
   { "-bg", 		"*background",	XrmoptionSepArg, NULL },
   { "-label", 		"*label",	XrmoptionSepArg, NULL },
   { "-passwd", 	"*passwd", 	XrmoptionSepArg, NULL },
   { "-system_modal", 	"*system_modal",XrmoptionNoArg, "False" },
   { "+system_modal", 	"*system_modal",XrmoptionNoArg, "True" },
   { "-one_try", 	"*one_try",	XrmoptionNoArg, "False" },
   { "+one_try", 	"*one_try",	XrmoptionNoArg, "True" },
   { "+set", 		"*set",		XrmoptionNoArg, "True" },
   { "-set", 		"*set",		XrmoptionNoArg, "False" },
};

#define XtNset "set"
#define XtCset "Set"

#define XtNlabel "label"
#define XtClabel "Label"

#define XtNpasswd "passwd"
#define XtCpasswd "Passwd"

#define XtNsystem_modal "system_modal"
#define XtCsystem_modal "System_modal"

#define XtNone_try "one_try"
#define XtCone_try "One_try"

typedef struct {
    String passwd;
    String label;
    Bool system_modal;
    Bool one_try;
    Bool set;
} AppData, *AppDataPtr;

static AppData app_data;

static XtResource resources[] = {
    {
        XtNset,
        XtCset,
        XtRBool,
        sizeof(Bool),
        XtOffset(AppDataPtr, set),
        XtRImmediate,
        (XtPointer) False,
    },
    {
        XtNone_try,
        XtCone_try,
        XtRBool,
        sizeof(Bool),
        XtOffset(AppDataPtr, one_try),
        XtRImmediate,
        (XtPointer) False,
    },
    {
        XtNsystem_modal,
        XtCsystem_modal,
        XtRBool,
        sizeof(Bool),
        XtOffset(AppDataPtr, system_modal),
        XtRImmediate,
        (XtPointer) False,
    },
    {
        XtNlabel,
        XtClabel,
        XtRString,
        sizeof(String),
        XtOffset(AppDataPtr, label),
        XtRString,
	"Passwd:"
    },
    {
        XtNpasswd,
        XtCpasswd,
        XtRString,
        sizeof(String),
        XtOffset(AppDataPtr, passwd),
        XtRString,
        NULL
    },
};

static void Usage()
{
    printf("Usage: xpasswd [-fg color] [-bg color] [-label label] [+/- system_modal]\n");
    printf("               [+/- one_try] [-passwd password_file] [+/- set]\n");
    exit (1);
}

void main(argc, argv)
int argc;
char *argv[];
{
    Widget toplevel, bb, text_w, rowcol;
    XtAppContext app;
    Arg args[10];
    int n;
    Display *dpy;
    extern void map_dialog_cb();

    toplevel = (Widget) XtVaAppInitialize((XtAppContext *) &app, (String) "XPasswd", 
	(XrmOptionDescList) options, (Cardinal) XtNumber(options), 
	(Cardinal) &argc, (String *) argv, 
	(String *) NULL, (String *) NULL);

    XtGetApplicationResources(toplevel, &app_data, resources, XtNumber(resources), NULL, 0);

    if (argc > 1) Usage();

    n = 0;

    if (app_data.system_modal) {
	XtSetArg(args[n], XmNdialogStyle, XmDIALOG_SYSTEM_MODAL); n++;
    }

    XtSetArg(args[n], XmNnoResize, True); n++;
    XtSetArg(args[n], XmNdefaultPosition, False); n++;

    bb = XmCreateBulletinBoardDialog(toplevel, "bb", args, n);

    XtAddCallback((Widget) bb, (String) XmNmapCallback, 
	(XtCallbackProc) map_dialog_cb, (XtPointer) NULL);


    rowcol = XtVaCreateManagedWidget("rowcol", xmRowColumnWidgetClass, bb,
	XmNorientation, XmHORIZONTAL, NULL);

    XtVaCreateManagedWidget(app_data.label, xmLabelGadgetClass, rowcol, NULL);

    text_w = XtVaCreateManagedWidget("text_w", xmTextWidgetClass, rowcol, NULL);

    XtAddCallback((Widget) text_w, (String) XmNmodifyVerifyCallback, 
	(XtCallbackProc) check_passwd, (XtPointer) NULL);

    XtAddCallback((Widget) text_w, (String) XmNactivateCallback, 
	(XtCallbackProc) check_passwd, (XtPointer) NULL);

    XtManageChild(bb);

    XtRealizeWidget(bb);
    XtAppMainLoop(app);
}

static XtCallbackProc check_passwd(text_w, unused, cbs)
Widget text_w;
XtPointer unused;
XmTextVerifyCallbackStruct *cbs;
{
    FILE *fp = NULL;
    char *new;
    char buf[80];
    int len;
    extern char *crypt();
#define SALT "ab"

    if (cbs->reason == XmCR_ACTIVATE) {
	if (app_data.passwd == NULL) exit (0); /* No passwd so let 'em slide */
	if (passwd == NULL) {
		if (app_data.one_try) exit (1); /* Wrong */
		return;
	}
	
	if (! app_data.set) {
	    fp = fopen(ExpandFileName(app_data.passwd), "r");
	    if (fp == NULL) exit (1); /* open failure */
	    fgets(buf, sizeof(buf), fp);
	    fclose(fp);

	    if (buf[strlen(buf)-1] == '\n') buf[strlen(buf)-1] = NULL;
	    if (!strcmp(buf, crypt(passwd, SALT))) exit (0); /* Correct ! */
	}
	else {
	    fp = fopen(ExpandFileName(app_data.passwd), "w");
	    if (fp == NULL) exit (1); /* open failure */
	    strcpy(buf, crypt(passwd, SALT));
	    fprintf(fp, "%s", buf);
	    fclose (fp);
	    exit (0); /* Done */
	}

	if (app_data.one_try) exit (1); /* Wrong */

	XBell(XtDisplayOfObject(text_w), 100);
	XmTextSetString(text_w, "");
	return;
    }

    if (cbs->text->ptr == NULL) { /* backspace */
	cbs->endPos = strlen(passwd); /* delete from here to end */
	passwd[cbs->startPos] = 0;
	return;
    }

    if (cbs->text->length > 1) {
	cbs->doit = False; /* don't allow paste ops */
	return;
    }

    new = XtMalloc(cbs->endPos + 2); /* new char + NULL */
    if (passwd) {
	strcpy(new, passwd);
	XtFree(passwd);
    }
    else new[0] = NULL;

    passwd = new;
    strncat(passwd, cbs->text->ptr, cbs->text->length);
    passwd[cbs->endPos + cbs->text->length] = 0;

    for (len = 0; len < cbs->text->length; len++) 
	cbs->text->ptr[len] = '*';
}
