/*
 * xs_sndrcv.c
 *
 *	Here's two routines that cooperating Xt clients can use
 *	to send a small (up to 80 bytes) message from a sender
 *	to a receiver.
 *
 * Created: 4/30/92 rem
 *
 *		
 Copyright (c) 1992 by Richard Mauri Trimble Navigation Ltd.

 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in
 supporting documentation.

 This file is provided AS IS with no warranties of any kind.  The author
 shall have no liability with respect to the infringement of copyrights,
 trade secrets or any patents by this file or any part thereof.  In no
 event will the author be liable for any lost revenue or profits or
 other special, indirect and consequential damages.
 */

#include <stdio.h>
#include <Xm/Xm.h>

typedef struct {
    void (*user_proc) ();
    XtPointer user_data;
    char key[80];
} RcvEventHandle;
    
typedef struct {
	union {
	    long l[20];
	    short s[40];
	    char b[80];
	} data;
	Window receiver_win;
} SndRcvData;

#define PROPERTY_NAME "XSSNDRCV"
#define PROPERTY_TYPE "XSSNDRCVTYPE"

extern char *getenv();

void xs_keyreceiver_done(dpy, key)
Display *dpy;
char *key;
{
    static SndRcvData *pd2 = NULL;
    Atom property, type, actual_type;
    unsigned long nitems = 0, bytes_after = 0;
    long offset = 0, length = 0;
    int actual_format;
    char nam[132];

    sprintf(nam, "%s_%s", PROPERTY_NAME, key);
    property = XInternAtom(dpy, nam, False);

    sprintf(nam, "%s_%s", PROPERTY_TYPE, key);
    type = XInternAtom(dpy, nam, False);
    length = sizeof(SndRcvData);

    XGetWindowProperty(dpy, DefaultRootWindow(dpy), property, offset, length, False, type, 
		&actual_type, &actual_format, &nitems, &bytes_after, &pd2);

    if (pd2) {
	pd2->receiver_win = 0;

	XChangeProperty(dpy, DefaultRootWindow(dpy), property, type, 8, PropModeReplace, (char *) pd2, (int) length);

	XFree(pd2);
	pd2 = NULL;
    }
}

static void keyevh(w, evhd, event)
Widget w;
RcvEventHandle *evhd;
XEvent *event;
{
	Display *dpy = XtDisplay(w);
	static SndRcvData *pd2;
	Atom property, type, actual_type;
	unsigned long nitems = 0, bytes_after = 0;
	long offset = 0, length = 0;
	int status = 0, actual_format;
	char nam[132];

	sprintf(nam, "%s_%s", PROPERTY_NAME, evhd->key);
	property = XInternAtom(dpy, nam, False);

	sprintf(nam, "%s_%s", PROPERTY_TYPE, evhd->key);
	type = XInternAtom(dpy, nam, False);
	length = sizeof(SndRcvData);

	if (pd2) {
	    XFree(pd2);
	    pd2 = NULL;
	}

	XGetWindowProperty(dpy, DefaultRootWindow(dpy), property, offset, length, False, type, 
		&actual_type, &actual_format, &nitems, &bytes_after, &pd2);

	if (pd2) (*evhd->user_proc) (pd2->data.b, evhd->user_data);
}

Bool xs_keyreceiver_rootdata(key, widget, data, datasize)
char *key;
Widget widget;
XtPointer data;
int datasize;
{
    char nam[132];
    Atom property, type, actual_type;
    Display *dpy = XtDisplay(widget);
    static RcvEventHandle evhd;
    static SndRcvData *pd2 = NULL;
    unsigned long nitems = 0, bytes_after = 0;
    long offset = 0, length = 0;
    int status = 0, actual_format;

    sprintf(nam, "%s_%s", PROPERTY_NAME, key);
    property = XInternAtom(dpy, nam, False);

    sprintf(nam, "%s_%s", PROPERTY_TYPE, key);
    type = XInternAtom(dpy, nam, False);
    length = sizeof(SndRcvData);

    XGetWindowProperty(dpy, DefaultRootWindow(dpy), property, offset, length, False, type, 
		&actual_type, &actual_format, &nitems, &bytes_after, &pd2);

    if (pd2 == NULL) return False;

    memcpy((char *) data, (char *) pd2->data.b, datasize);

    XFree(pd2);
    pd2 = NULL;
    return True;
}

void xs_keyreceiver(key, widget, user_proc, user_data)
char *key;
Widget widget;
void (*user_proc) ();
XtPointer user_data;
{
    Display *dpy = XtDisplay(widget);
    static Bool first = True;
    static RcvEventHandle evhd;
    static SndRcvData pd, *pd2 = NULL;
    Atom property, type, actual_type;
    unsigned long nitems = 0, bytes_after = 0;
    long offset = 0, length = 0;
    int status = 0, actual_format;
    char nam[132];

    if (first) first = False;
    else XtRemoveEventHandler(widget, PropertyChangeMask, False, keyevh, &evhd);

    sprintf(nam, "%s_%s", PROPERTY_NAME, key);
    property = XInternAtom(dpy, nam, False);

    sprintf(nam, "%s_%s", PROPERTY_TYPE, key);
    type = XInternAtom(dpy, nam, False);
    length = sizeof(SndRcvData);

    XGetWindowProperty(dpy, DefaultRootWindow(dpy), property, offset, length, False, type, 
		&actual_type, &actual_format, &nitems, &bytes_after, &pd2);

    if (pd2 == NULL) {
	    pd.receiver_win = XtWindow(widget);

	    XChangeProperty(dpy, DefaultRootWindow(dpy), property, type, 8, 
		PropModeReplace, (char *) &pd, (int) length);
    }
    else {
	pd2->receiver_win = XtWindow(widget);

	XChangeProperty(dpy, DefaultRootWindow(dpy), property, type, 8, PropModeReplace, (char *) pd2, (int) length);

	XFree(pd2);
	pd2 = NULL;
    }

    evhd.user_proc = user_proc;
    evhd.user_data = user_data;
    strcpy(evhd.key, key);

    XtAddEventHandler(widget, PropertyChangeMask, False, keyevh, &evhd);
}

void xs_keysender(key, dpy, data)
char *key;
Display *dpy;
XtPointer data;
{
	static SndRcvData pd, *pd2 = NULL;
	Atom property, type, actual_type;
	unsigned long nitems = 0, bytes_after = 0;
	long offset = 0, length = 0;
	int status = 0, actual_format;
	char nam[132];

	sprintf(nam, "%s_%s", PROPERTY_NAME, key);
	property = XInternAtom(dpy, nam, False);

	sprintf(nam, "%s_%s", PROPERTY_TYPE, key);
	type = XInternAtom(dpy, nam, False);
	length = sizeof(SndRcvData);

	XGetWindowProperty(dpy, DefaultRootWindow(dpy), property, offset, length, False, type, 
		&actual_type, &actual_format, &nitems, &bytes_after, &pd2);

	if (pd2 == NULL) {
	    /* Create it and leave. No receiver yet */
	    memcpy(pd.data.b, (char *) data, sizeof(pd.data));

	    XChangeProperty(dpy, DefaultRootWindow(dpy), property, type, 8,
		PropModeReplace, (char *) &pd, (int) length);
                return;
	}

	if (pd2->receiver_win == NULL) {
	    XFree(pd2);
	    pd2 = NULL;
	    return; /* receiver is not there so do nothing */
	}
	else {
	    /*
	     * Pass the data to the root window property
	     */
	    memcpy(pd2->data.b, (char *) data, sizeof(pd.data));

	    XChangeProperty(dpy, DefaultRootWindow(dpy), property, type, 8,
		PropModeReplace, (char *) pd2, (int) length);
	    /*
	     * This will cause PropertyNotify event to the receiver
	     */
	    XChangeProperty(dpy, pd2->receiver_win, property, type, 8,
		PropModeReplace, (char *) pd2, (int) length);
	}

	XFree(pd2);
}

void xs_GetPointerLocation(w, x, y)
Widget w;
int *x, *y;
{
    Window root_win, child_win;
    int root_x, root_y, win_x, win_y, key_btns;

    XQueryPointer(XtDisplayOfObject(w), XtWindowOfObject(w), &root_win, 
	&child_win, x, y, &win_x, &win_y, &key_btns);
}

void xs_PositionShellAtPointer(w)
Widget w;
{
    Dimension width = 0, height = 0;
    int x, y, x_root, y_root;
    int screen_width, screen_height;

    screen_width = WidthOfScreen(XtScreenOfObject(w));
    screen_height = HeightOfScreen(XtScreenOfObject(w));

    xs_GetPointerLocation(w, &x, &y);

    XtVaGetValues(w, XmNwidth, &width, XmNheight, &height, NULL);

    if (x - width/2 < 0) x_root = width/2;
    else if (x + width/2 > screen_width) x_root = screen_width - width/2;
    else x_root = x - width/2;

    if (y - height/2 < 0) y_root = height/2;
    else if (y + height/2 > screen_height) y_root = screen_height - height/2;
    else y_root = y - height/2;

    XtVaSetValues(w, XmNx, x_root, XmNy, y_root, NULL);
}

void map_dialog_cb(w, cd, cbs)
Widget w;
XtPointer cd;
XmAnyCallbackStruct *cbs;
{
    xs_PositionShellAtPointer(w);
}

char *ExpandFileName( fname )
char *fname;
{
    char        *eptr;
    char        *fptr;
    char        *gptr;
    static 	char tname[256];
    static	char element[256];

    /*
     * Set up to build translated file name.  If name starts with
     * a slash, copy that now.
     */
    fptr = fname;
    tname[0] = '\0';
    if(*fptr == '/') {
        strcat(tname, "/");
        fptr++;
    }

    /*
     * Scan the file name for each piece between slashes.  Look up each
     * piece in the environment database via getenv().  If there is a
     * match, replace the piece with the match.  Pieces that start with
     * a $ must translate or error...
     */
    while(*fptr != '\0') {
        eptr = element;
        while((*fptr != '/') && (*fptr != '\0'))
            *eptr++ = *fptr++;
        *eptr++ = '\0';
        if (element[0] == '$') {
                gptr = getenv(&element[1]);
                if (gptr == NULL) return NULL;
        }
        else {
                gptr = getenv(element);
                if (gptr == NULL || *gptr == '\0') gptr = element;
        }
        strcat(tname, gptr);
        if(*fptr == '/') {
            strcat(tname, "/");
            fptr++;
        }
    }

    return tname;
}
